@@ -4,7 +4,7 @@
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2010 Aleksey Sanin <aleksey@aleksey.com>
+ * Copyright (C) 2010-2016 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 #include "globals.h"
 
@@ -1082,6 +1082,11 @@ xmlSecGCryptKeyDataDsaXmlRead(xmlSecKeyDataId id,
         goto done;
     }
 
+    /* Convert from OpenSSL parameter ordering to the OpenPGP order. */
+    /* First check that x < y; if not swap x and y  */
+    if((x != NULL) && (gcry_mpi_cmp (x, y) > 0)) {
+        gcry_mpi_swap (x, y);
+    }
 
     /* construct pub/priv key pairs */
     err = gcry_sexp_build(&pub_key, NULL,
